/******************************************************************************
** FILE         : ix_gen.ec
** DATE         : 22/09/1998
** Version      : 1.00.1a1
**
** DESCRIPTION  : ESQL/C interface functions for 4GL compiler
**                usable with ESQL/C Version 7.x or higher
** TODO         : improve return of functions.
**  WARNING! WARNING! DON'T MODIFY THIS SOURCE ELSE YOUR PCODE RUNNER COULD NOT
**  BE RUN.
******************************************************************************/

/******************************************************************************
**                           INCLUSIONS + MACROS
******************************************************************************/ 

EXEC SQL include sqlda;
EXEC SQL include sqlca;

/* fetch mode */
enum fetchmode {
  FNEXT=1,
  FPREVIOUS,
  FFIRST,
  FLAST,
  FCURRENT,
  FABSOLUTE,
  FRELATIVE 
};

/******************************************************************************
**                              FUNCTIONS
******************************************************************************/

/*******************************************************************************
** FUNCTION ixgenPrepare : prepare a statement
**                                 
** Parameters :
**  stmtName : name of the cursor
**  cmd      : SQL command string
** Returnings :
**  none
*******************************************************************************/

void ixgenPrepare(stmtName,cmd)
EXEC SQL BEGIN DECLARE SECTION;
PARAMETER char *stmtName;
PARAMETER char *cmd;
EXEC SQL END DECLARE SECTION;
{
 EXEC SQL PREPARE :stmtName FROM :cmd;
}

/*******************************************************************************
** FUNCTION ixgenDeclareForStmt : declare a cursor with a statement name
**                                 
** Parameters :
**  cuName   : name of the cursor
**  stmtName : name of the statement
**  scroll   : 1 if it is a scroll cursor
**  withHold : 1 if it is a cursor with Hold
** Returnings :
**  none
*******************************************************************************/

void ixgenDeclareForStmt(cuName,stmtName,scroll,withHold)
EXEC SQL BEGIN DECLARE SECTION;
PARAMETER char *cuName;
PARAMETER char *stmtName;
EXEC SQL END DECLARE SECTION;
int scroll;
int withHold;
{
  if(scroll){
    if(withHold){
      EXEC SQL DECLARE :cuName SCROLL CURSOR WITH HOLD FOR :stmtName;
    }else{
      EXEC SQL DECLARE :cuName SCROLL CURSOR FOR :stmtName;
    }
  }else{
    if(withHold){
      EXEC SQL DECLARE :cuName CURSOR WITH HOLD FOR :stmtName;
    }else{
      EXEC SQL DECLARE :cuName CURSOR FOR :stmtName;
    }
  }
}

/*******************************************************************************
** FUNCTION ixgenDescribe : describe prepared statement into sqlda variable
**                                 
** Parameters :
**  stmtName : name of the cursor
**  intoVar  : sqlda structure pointer
** Returnings :
**  none
*******************************************************************************/

void ixgenDescribe(stmtName,intoSqlda)
EXEC SQL BEGIN DECLARE SECTION;
PARAMETER char *stmtName;
PARAMETER struct sqlda **intoSqlda;
EXEC SQL END DECLARE SECTION;
{
/* DESCRIBE statement uses pointer address of a sqlda structure.
   But ESQL/C doesn't accept next syntax: *mySqlda or mySqlda[0] */
#define mySqldaDescribe (intoSqlda[0])
 EXEC SQL DESCRIBE :stmtName INTO mySqldaDescribe;
}

/*******************************************************************************
** FUNCTION ixgenExecute : execute a statement
**                                 
** Parameters :
**  stmtName : name of the cursor
**  useVar   : host variables for the using descriptor
**  intoVar  : host variables for the into descriptor
** Returnings :
**  none
*******************************************************************************/

void ixgenExecute(stmtName,useVar,intoVar)
EXEC SQL BEGIN DECLARE SECTION;
PARAMETER char *stmtName;
EXEC SQL END DECLARE SECTION;
struct sqlda *useVar;
struct sqlda *intoVar;
{
 if(useVar&&useVar->sqld){
   if(intoVar&&intoVar->sqld){
     EXEC SQL EXECUTE :stmtName INTO DESCRIPTOR intoVar USING DESCRIPTOR useVar;
   } else {
     EXEC SQL EXECUTE :stmtName USING DESCRIPTOR useVar;
   }
 } else {
   if(intoVar&&intoVar->sqld){
     EXEC SQL EXECUTE :stmtName INTO DESCRIPTOR intoVar;
   } else {
     EXEC SQL EXECUTE :stmtName;
   }
 }
}

/*******************************************************************************
** FUNCTION ixgenExecuteImmediate : execute a "immediate" Sql command string
** Parameters :
**  cmd      : SQL command string
** Returnings :
**  none
*******************************************************************************/

void ixgenExecuteImmediate(cmd)
EXEC SQL BEGIN DECLARE SECTION;
PARAMETER char *cmd;
EXEC SQL END DECLARE SECTION;
{
 EXEC SQL EXECUTE IMMEDIATE :cmd;
}

/*******************************************************************************
** FUNCTION ixgenOpen : open a cursor
** Parameters :
**  cuName   : name of the cursor
**  useVar   : host variables for the using descriptor
**  wr       : 1 if the cursor is open with reoptimization
** Returnings :
**  none
*******************************************************************************/

void ixgenOpen(cuName,useVar,wr)
EXEC SQL BEGIN DECLARE SECTION;
PARAMETER char *cuName;
PARAMETER struct sqlda *useVar;
EXEC SQL END DECLARE SECTION;
int wr;
{
 if(useVar&&useVar->sqld){
   if(wr){
     EXEC SQL OPEN :cuName USING DESCRIPTOR useVar WITH REOPTIMIZATION;
   } else {
     EXEC SQL OPEN :cuName USING DESCRIPTOR useVar;
   }
 } else {
   if(wr){
     EXEC SQL OPEN :cuName WITH REOPTIMIZATION;
   } else {
     EXEC SQL OPEN :cuName;
   }
 }
}

/*******************************************************************************
** FUNCTION ixgenFetch : fetch a cursor
** Parameters :
**  cmd      : SQL command string
**  intoVar  : host variables for the into descriptor
**  how      : kind of iteration (only for scroll cursor)
**  value    : value of iteration
** Returnings :
**  none
*******************************************************************************/

void ixgenFetch(cuName,intoVar,how,value)
EXEC SQL BEGIN DECLARE SECTION;
PARAMETER char *cuName;
PARAMETER struct sqlda *intoVar;
int how;
int value;
EXEC SQL END DECLARE SECTION;
{
 if(intoVar&&intoVar->sqld){
   switch(how) {
     case FNEXT:
       EXEC SQL FETCH NEXT :cuName USING DESCRIPTOR intoVar;
       break;
     case FPREVIOUS:
       EXEC SQL FETCH PREVIOUS :cuName USING DESCRIPTOR intoVar;
       break;
     case FFIRST:
       EXEC SQL FETCH FIRST :cuName USING DESCRIPTOR intoVar;
       break;
     case FLAST:
       EXEC SQL FETCH LAST :cuName USING DESCRIPTOR intoVar;
       break;
     case FCURRENT:
       EXEC SQL FETCH CURRENT :cuName USING DESCRIPTOR intoVar;
       break;
     case FABSOLUTE:
       EXEC SQL FETCH ABSOLUTE :value :cuName USING DESCRIPTOR intoVar;
       break;
     case FRELATIVE:
       EXEC SQL FETCH RELATIVE :value :cuName USING DESCRIPTOR intoVar;
       break;
     default:
       EXEC SQL FETCH :cuName USING DESCRIPTOR intoVar;
       break;
   }
 } else {
   switch(how) {
     case FNEXT:
       EXEC SQL FETCH NEXT :cuName;
       break;
     case FPREVIOUS:
       EXEC SQL FETCH PREVIOUS :cuName;
       break;
     case FFIRST:
       EXEC SQL FETCH FIRST :cuName;
       break;
     case FLAST:
       EXEC SQL FETCH LAST :cuName;
       break;
     case FCURRENT:
       EXEC SQL FETCH CURRENT :cuName;
       break;
     case FABSOLUTE:
       EXEC SQL FETCH ABSOLUTE :value :cuName;
       break;
     case FRELATIVE:
       EXEC SQL FETCH RELATIVE :value :cuName;
       break;
     default:
       EXEC SQL FETCH :cuName;
       break;
   }
 }
}

/*******************************************************************************
** FUNCTION ixgenClose : close a cursor
** Parameters :
**  cuName    : name of the cursor
** Returnings :
**  none
*******************************************************************************/

void ixgenClose(cuName)
EXEC SQL BEGIN DECLARE SECTION;
PARAMETER char *cuName;
EXEC SQL END DECLARE SECTION;
{
 EXEC SQL CLOSE :cuName;
}

/*******************************************************************************
** FUNCTION ixgenFree : free a cursor
** Parameters :
**  cuName    : name of the cursor
** Returnings :
**  none
*******************************************************************************/

void ixgenFree(cuName)
EXEC SQL BEGIN DECLARE SECTION;
PARAMETER char *cuName;
EXEC SQL END DECLARE SECTION;
{
 EXEC SQL FREE :cuName;
}

/*******************************************************************************
** FUNCTION ixgenPut : Put into a cursor
** Parameters :
**  cuName   : name of the cursor
**  useVar   : host variables for the using descriptor
** Returnings :
**  none
*******************************************************************************/

void ixgenPut(cuName,useVar)
EXEC SQL BEGIN DECLARE SECTION;
PARAMETER char *cuName;
PARAMETER struct sqlda *useVar;
EXEC SQL END DECLARE SECTION;
{
 EXEC SQL PUT :cuName USING DESCRIPTOR useVar;
}

/*******************************************************************************
** FUNCTION ixgenFlush : Flush a cursor
** Parameters :
**  cuName    : name of the cursor
** Returnings :
**  none
*******************************************************************************/

void ixgenFlush(cuName)
EXEC SQL BEGIN DECLARE SECTION;
PARAMETER char *cuName;
EXEC SQL END DECLARE SECTION;
{
 EXEC SQL FLUSH :cuName;
}

/*******************************************************************************
** FUNCTION ixgenBeginWork : Begin Work Statement
** Parameters :
**  none
** Returnings :
**  none
*******************************************************************************/

void ixgenBeginWork()
{
 EXEC SQL BEGIN WORK;
}

/*******************************************************************************
** FUNCTION ixgenCommitWork : Commit Work Statement
** Parameters :
**  none
** Returnings :
**  none
*******************************************************************************/

void ixgenCommitWork()
{
 EXEC SQL COMMIT WORK;
}

/*******************************************************************************
** FUNCTION ixgenRollbackWork : Rollback Work Statement
** Parameters :
**  none
** Returnings :
**  none
*******************************************************************************/

void ixgenRollbackWork()
{
 EXEC SQL ROLLBACK WORK;
}
